#include <Arduino.h>
#include <HID-Project.h>

uint8_t const pin_lr{2};
uint8_t const pin_rr{A3};
uint8_t const pin_left{5};
uint8_t const pin_up{3};
uint8_t const pin_down{7};
uint8_t const pin_right{6};
uint8_t const pin_y{A2};
uint8_t const pin_x{15};
uint8_t const pin_b{A0};
uint8_t const pin_a{14};
uint8_t const pin_select{9};
uint8_t const pin_start{10};

uint8_t const ButtonNum{8};
uint8_t const ButtonPin[ButtonNum] = {pin_lr, pin_rr, pin_y,      pin_x,
                                      pin_b,  pin_a,  pin_select, pin_start};
uint8_t const ButtonKey[ButtonNum] = {11, 12, 13, 14, 15, 16, 17, 18};
bool buttonPreviousState[ButtonNum];

enum DPad_e { DPadUp = 0, DPadLeft = 1, DPadRight = 2, DPadDown = 3 };

uint8_t const DPadNum{DPadDown + 1};
uint8_t const DPadPin[DPadNum] = {pin_up, pin_left, pin_right, pin_down};
bool dPadState[DPadNum];

void setup() {
  for (size_t i = 0; i < ButtonNum; ++i) {
    pinMode(ButtonPin[i], INPUT_PULLUP);
    buttonPreviousState[i] = digitalRead(ButtonPin[i]);
  }

  for (size_t i = 0; i < DPadNum; ++i) {
    pinMode(DPadPin[i], INPUT_PULLUP);
    dPadState[i] = digitalRead(DPadPin[i]);
  }

  Gamepad.begin();
}

void loop() {
  for (size_t i = 0; i < ButtonNum; ++i) {
    bool buttonCurrentState = digitalRead(ButtonPin[i]);
    if (buttonCurrentState != buttonPreviousState[i]) {
      if (buttonCurrentState == LOW) {
        Gamepad.press(ButtonKey[i]);
      } else {
        Gamepad.release(ButtonKey[i]);
      }
    }
    buttonPreviousState[i] = buttonCurrentState;
  }

  for (size_t i = 0; i < DPadNum; ++i) {
    bool dPadCurrentState = digitalRead(DPadPin[i]);
    dPadState[i] = dPadCurrentState;
  }

  if (dPadState[DPadUp] != dPadState[DPadDown]) {
    if (dPadState[DPadUp] == LOW) {
      if (dPadState[DPadLeft] == dPadState[DPadRight]) {
        Gamepad.dPad1(GAMEPAD_DPAD_UP);
      } else if (dPadState[DPadLeft] == LOW) {
        Gamepad.dPad1(GAMEPAD_DPAD_UP_LEFT);
      } else {
        Gamepad.dPad1(GAMEPAD_DPAD_UP_RIGHT);
      }
    } else {
      if (dPadState[DPadDown] == LOW) {
        if (dPadState[DPadLeft] == dPadState[DPadRight]) {
          Gamepad.dPad1(GAMEPAD_DPAD_DOWN);
        } else if (dPadState[DPadLeft] == LOW) {
          Gamepad.dPad1(GAMEPAD_DPAD_DOWN_LEFT);
        } else {
          Gamepad.dPad1(GAMEPAD_DPAD_DOWN_RIGHT);
        }
      }
    }
  } else if (dPadState[DPadLeft] != dPadState[DPadRight]) {
    if (dPadState[DPadLeft] == LOW) {
      Gamepad.dPad1(GAMEPAD_DPAD_LEFT);
    } else {
      Gamepad.dPad1(GAMEPAD_DPAD_RIGHT);
    }
  } else {
    Gamepad.dPad1(GAMEPAD_DPAD_CENTERED);
  }

  Gamepad.write();
}